<?php
/**
  Minifies and combines css files
*/

# get correct id for plugin
$thisfile = basename(__FILE__, '.php');

# register plugin
register_plugin(
	$thisfile, 
	'Minify CSS', 	
	'0.1', 		
	'Astrid Hanssen',
	'http://astrid-hanssen.de', 
	'Minification of one or more CSS files',
	'',
	''  
);

/**
 * Minifies one or more .css files to a common .min.css file.
 * Just list the .css files to minify as parameters.
 *
 * Output file name is derived from the last file name parameter, supplemented with ".min.css".
 * This last file does not need to exist, so you can minify 'css/style1.css' and 'css/style2.css' to 'css/common.min.css'
 * through calling return_minify_css('css/style1.css', 'css/style2.css', 'css/common').
 *
 * css file names can be:
 * starting with http:// or https://, this can be a local or a remote css file
 *  (dont use remote url as last parameter, because the output file
 *  is derived from this, so add a local name parameter at last), 
 * starting with '/', this is a file either relative to the local site root
 *  or to the root path in GSROOTPATH
 * otherwise, the file path is relative to the current theme
 */
function return_minify_css() {
  $inputCssFiles = func_get_args();
  if (is_array($inputCssFiles[0])) {
	  $inputCssFiles = $inputCssFiles[0];
  }
  $numCssFiles = count($inputCssFiles);
  if (!$numCssFiles) return '';
  $targetf = $inputCssFiles[$numCssFiles-1];
  if (!strcasecmp(substr($targetf, -4), '.css')) {
	  $targetf = substr($targetf,0,strrpos($targetf,'.')).'.min.css';
  } else {
	  $targetf = $targetf.'.min.css';
  }
  $targetFile = _mk_minify_css_absfile_path($targetf);
  $targetCssUrl = _mk_minify_css_output_url($targetFile);
  $targetTime = (file_exists($targetFile))? @filemtime($targetFile) : 0;

  $cssFiles = [];
  $doMinimize = (!($targetTime))? true : false;
  foreach ($inputCssFiles as $cssf) {
	  $cssFile = _mk_minify_css_absfile_path($cssf);
	  if (file_exists($cssFile)) {
		  $doMinimize |= (filemtime($cssFile) > $targetTime)? true : false;
		  $cssFiles[] = $cssFile;
	  } else if ((substr($cssFile,0,7) === 'http://') || (substr($cssFile,0,8) === 'https://')) {
		  //  file on external domain, cannot know whether it is new
		  $cssFiles[] = $cssFile;
	  }
  }

  if ($doMinimize) {
	$buffer = '';
	foreach ($cssFiles as $cssFile) {
	  $buffer .= file_get_contents($cssFile);
	}
	$buffer = preg_replace('#/\*((?!\*/).)*\*/#s', '', $buffer);
	$buffer = str_replace(["\r\n", "\r", "\n", "\t"], ' ', $buffer);
	$buffer = preg_replace('#\s{2,}#', ' ', $buffer);
	$buffer = preg_replace('#\s*([:;,{}])\s*#', '$1', $buffer);
	$buffer = preg_replace('#([ :])0\.#', '$1.', $buffer);
	$buffer = preg_replace('#;}#', '}', $buffer);
    file_put_contents($targetFile, $buffer);
 }

  return $targetCssUrl;

}

/**
 * outputs the resulting CSS file name, generated by return_minify_css.
 */
function get_minify_css() {
  $inputCssFiles = func_get_args();
  if (is_array($inputCssFiles[0])) {
	  $inputCssFiles = $inputCssFiles[0];
  }
  echo call_user_func_array('return_minify_css', $inputCssFiles);
}


/************************** Helpers *******************************/

function _mk_minify_css_absfile_path($inputFile) {
  global $SITEURL, $TEMPLATE;
    $GSROOTPATH = GSROOTPATH;
	if ((substr($inputFile,0,7) === 'http://') || (substr($inputFile,0,8) === 'https://')) {
		$slen = strlen($SITEURL);
		if (!strcmp(substr($inputFile, 0, $slen), $SITEURL)) { // '===' fails
			return $GSROOTPATH.substr($inputFile, $slen);
		} else {
			// file on external domain, file_get_contents() can probably work with this
			return $inputFile;
		}
	}
	if (substr($inputFile,0,1) === '/') {
		if (!strcmp(substr($inputFile, 0, strlen($GSROOTPATH)), $GSROOTPATH)) {
			return $inputFile;
		}
		return $GSROOTPATH.$inputFile;
	}
	return GSTHEMESPATH.$TEMPLATE.'/'.$inputFile;
}


function _mk_minify_css_output_url($absFile) {
  global $SITEURL;
    $GSROOTPATH = GSROOTPATH;
    $slen = strlen($GSROOTPATH);
	if (!strcmp(substr($absFile, 0, $slen), $GSROOTPATH)) {
		return $SITEURL.substr($absFile, $slen);
	}
	return $absFile; 
}